import requests

api_url = 'https://api.pitcher.com'
pitcher_file_categories = ['presentation', 'zip', 'interface']


def request(method, url, token=None, **kwargs):
    return getattr(requests, method)(
        api_url + url,
        headers={
            'Authorization': 'Token %s' % token
        } if token else None,
        **kwargs
    )


def authenticate(username, password):
    res = request(
        'post',
        '/auth/user/',
        json={
            'username': username,
            'password': password
        }
    )

    if res.status_code != 200:
        raise ValueError

    return res.json().get('token')



import os
import time
import zipfile
from os import mkdir
from os.path import exists, join

from click.testing import CliRunner

from kudu.__main__ import cli
from kudu.api import authenticate
from kudu.api import request as api_request
from kudu.commands.push import CATEGORY_RULES
from kudu.config import write_config
from kudu.mkztemp import NameRule, mkztemp


def test_interface():
    runner = CliRunner()
    with runner.isolated_filesystem():
        write_config({'file_id': 1105408})

        mkdir('interface')

        with open('interface/index.html', 'a') as f:
            f.write('<html></html>')

        result = runner.invoke(cli, ['push'])
        assert result.exit_code == 0


def test_rules():
    runner = CliRunner()

    with runner.isolated_filesystem():
        mkdir('interface')
        open('interface/index.html', 'a').close()

        rules = [r.rule for r in CATEGORY_RULES if '' in r.category]
        _, name = mkztemp('interface_test', name_rules=rules)

        zf = zipfile.ZipFile(name)
        namelist = zf.namelist()
        assert namelist == ['interface_test/index.html']

    with runner.isolated_filesystem():
        open('index.html', 'a').close()
        open('thumbnail.png', 'a').close()

        rules = [r.rule for r in CATEGORY_RULES if 'zip' in r.category]
        _, name = mkztemp('test', name_rules=rules)

        zf = zipfile.ZipFile(name)
        namelist = zf.namelist()
        assert len(namelist) == 2
        assert 'test/test.png' in namelist
        assert 'test/index.html' in namelist


def test_zip():
    runner = CliRunner()
    with runner.isolated_filesystem():
        open('index.html', 'a').close()
        open('thumbnail.png', 'a').close()
        result = runner.invoke(cli, ['push', '-f', 519655])
        assert result.exit_code == 0


def test_json():
    runner = CliRunner()
    with runner.isolated_filesystem():
        open('upload.json', 'a').close()
        result = runner.invoke(cli, ['push', '-f', 703251, '-p', 'upload.json'])
        assert result.exit_code == 0



import os
import re
import subprocess

import click


def default_pitcher_folders():
    click.echo(
        'Your Pitcher Folders are configured automatically based on your iOS simulator devices. Please check '
        'that they are accurate. You can suppress this message by setting them explicitly. Run the following '
        'command and follow the instructions in your editor to edit your configuration file:\n\n'
        '\tkudu config --global --edit\n'
    )

    ios_sim_devices = []

    try:
        instruments_output = subprocess.check_output([
            'instruments', '-s', 'devices'
        ]).splitlines()
        prog = re.compile(r"iPad .+\[([^\]]+)")
        devices_path = os.path.expanduser(
            '~/Library/Developer/CoreSimulator/Devices/'
        )

        for line in instruments_output:
            prog_match = prog.match(line)
            if prog_match:
                device_guid = prog_match.group(1)
                find_output = subprocess.check_output([
                    'find',
                    os.path.join(devices_path, device_guid), '-name',
                    'Pitcher Folders'
                ]).strip()
                if find_output:
                    ios_sim_devices.append({
                        'name': line,
                        'guid': device_guid,
                        'pitcher_folders': find_output
                    })
    except OSError:
        pass

    if len(ios_sim_devices) > 0:
        ios_sim_device_index = 0

        if len(ios_sim_devices) > 1:
            for key, device in enumerate(ios_sim_devices):
                click.echo('%d. %s' % (key + 1, device['name']))
            ios_sim_device_index = int(
                click.prompt(
                    'Please select a device',
                    type=click.Choice([
                        str(key + 1)
                        for key, device in enumerate(ios_sim_devices)
                    ])
                )
            ) - 1

        return ios_sim_devices[ios_sim_device_index]['pitcher_folders']



from os.path import exists, join
from zipfile import ZipFile

from click.testing import CliRunner

from kudu.__main__ import cli
from kudu.config import write_config


def test_interface():
    runner = CliRunner()
    with runner.isolated_filesystem():
        write_config({'file_id': 527702})
        result = runner.invoke(cli, ['pull'])
        assert result.exit_code == 0
        assert exists(join('interface', 'index.html'))
        assert exists(join('interface_1', 'index.html'))
        assert exists(join('lang', 'AR.csv'))


def test_json_path():
    runner = CliRunner()
    with runner.isolated_filesystem():
        result = runner.invoke(cli, ['pull', '-f', 519631, '-p', 'test.json'])
        assert result.exit_code == 0
        assert exists('test.json')


def test_zip():
    runner = CliRunner()
    with runner.isolated_filesystem():
        write_config({'file_id': 519629})
        result = runner.invoke(cli, ['pull'])
        assert result.exit_code == 0
        assert exists('index.html')
        assert exists('thumbnail.png')
        assert exists(join('folder', 'foobar.json'))


def test_zip_path():
    runner = CliRunner()
    with runner.isolated_filesystem():
        result = runner.invoke(cli, ['pull', '-f', 519629, '-p', 'test.zip'])
        assert result.exit_code == 0
        assert exists('test.zip')

        zip_file = ZipFile('test.zip')
        assert zip_file.testzip() is None



import os
import shutil
import tempfile
import time
from os import mkdir, getcwd

from click.testing import CliRunner
from watchdog import events

from kudu.commands import link


def test_interface_path_converter():
    dst = os.path.join(tempfile.gettempdir(), str(time.time()))
    mkdir(dst)

    runner = CliRunner()
    with runner.isolated_filesystem():
        src = getcwd()

        mkdir('interface')
        open(os.path.join('interface', 'test.html'), 'wb').close()

        link.copyfiles(
            src, dst,
            link.InterfacePathConverter({'filename': 'interface_xy.zip'})
        )
        assert os.path.exists(os.path.join(dst, 'interface_xy', 'test.html'))

    shutil.rmtree(dst)


def test_presentation_path_converter():
    dst = os.path.join(tempfile.gettempdir(), str(time.time()))
    mkdir(dst)

    runner = CliRunner()
    with runner.isolated_filesystem():
        src = getcwd()

        open('index.html', 'wb').close()
        open('thumbnail.png', 'wb').close()
        mkdir('iPadOnly')
        open(os.path.join('iPadOnly', 'iPad.html'), 'wb').close()

        link.copyfiles(
            src, dst,
            link.PresentationPathConverter({'filename': '1234_4321.zip'})
        )
        assert os.path.exists(
            os.path.join(dst, 'slides', '1234_4321', 'index.html')
        )
        assert os.path.exists(
            os.path.join(dst, 'slides', '1234_4321', '1234_4321.png')
        )
        assert os.path.exists(
            os.path.join(dst, 'slides', '1234_4321', 'iPad.html')
        )

    shutil.rmtree(dst)


def test_copy_files_event_handler():
    dst = os.path.join(tempfile.gettempdir(), str(time.time()))
    mkdir(dst)

    runner = CliRunner()
    with runner.isolated_filesystem():
        src = getcwd()
        converter = link.PresentationPathConverter({
            'filename': '1234_4321.zip'
        })
        event_handler = link.CopyFilesEventHandler(src, dst, converter)

        class TestEvent(object):
            event_type = events.EVENT_TYPE_MODIFIED
            src_path = os.path.join(src, 'index.html')
            is_directory = False

        open('index.html', 'wb').close()
        event_handler.on_any_event(TestEvent)

        assert os.path.exists(
            os.path.join(dst, 'slides', '1234_4321', 'index.html')
        )

    shutil.rmtree(dst)



import os
import re
import tempfile
import zipfile


class NameRule:

    def __init__(self, pattern, path):
        if not isinstance(pattern, str):
            pattern = os.path.join(*pattern)
        self.pattern = re.compile(pattern)

        if not isinstance(path, str):
            path = os.path.join(*path)
        self.path = path


def mkztemp(base_name, root_dir=None, base_dir=None, name_rules=None):
    save_cwd = os.getcwd()

    if root_dir is not None:
        os.chdir(root_dir)

    if base_dir is None:
        base_dir = os.curdir

    try:
        tmp_fd, tmp_name = tempfile.mkstemp('.zip')
        zf = zipfile.ZipFile(tmp_name, 'w', zipfile.ZIP_DEFLATED)

        for root, dirs, files in os.walk(base_dir):
            arcroot = os.path.relpath(root)

            for name in files:
                if arcroot == os.curdir:
                    arcname = name
                else:
                    arcname = os.path.join(arcroot, name)

                if name_rules:
                    for rule in name_rules:
                        m = rule.pattern.match(arcname)
                        if m:
                            arcname = rule.path.format(
                                *m.groups(), base_name=base_name
                            )

                filename = os.path.join(root, name)
                zf.write(filename, arcname)
    finally:
        if root_dir is not None:
            os.chdir(save_cwd)

    return tmp_fd, tmp_name



#!/usr/bin/env python

import click

from kudu.api import authenticate
from kudu.commands.init import init
from kudu.commands.link import link
from kudu.commands.pull import pull
from kudu.commands.push import push
from kudu.config import ConfigOption


@click.group()
@click.option(
    '--username', '-u', cls=ConfigOption, prompt=True, envvar='KUDU_USERNAME'
)
@click.option(
    '--password',
    '-p',
    cls=ConfigOption,
    prompt=True,
    hide_input=True,
    envvar='KUDU_PASSWORD'
)
@click.option('--token', '-t', cls=ConfigOption, envvar='KUDU_TOKEN')
@click.pass_context
def cli(ctx, username, password, token):
    if not token:
        try:
            token = authenticate(username, password)
        except ValueError:
            click.echo('Invalid username or password', err=True)
            exit(1)

    ctx.obj = {'username': username, 'password': password, 'token': token}


cli.add_command(init)
cli.add_command(pull)
cli.add_command(push)
cli.add_command(link)

if __name__ == '__main__':
    cli()



import click

from kudu.api import request
from kudu.config import write_config


@click.command()
@click.pass_context
def init(ctx):
    if click.confirm('Would you like to create a new file?'):
        file_id = create_file(ctx.obj['token'])
    else:
        file_id = validate_file(
            click.prompt('File ID', type=int), ctx.obj['token']
        )

    write_config({'file_id': file_id})


def create_file(token):
    app_id = click.prompt('Instance ID', type=int)
    file_body = click.prompt('File Body')

    res = request(
        'post',
        '/files/',
        json={
            'app':
                app_id,
            'body':
                file_body,
            'downloadUrl':
                'https://admin.pitcher.com/downloads/Pitcher%20HTML5%20Folder.zip'
        },
        token=token
    )
    json = res.json()

    if res.status_code != 201:
        if json.get('app'):
            click.echo('Invalid instance', err=True)
        else:
            click.echo('Unknown error', err=True)
        exit(1)

    return json.get('id')


def validate_file(file_id, token):
    api_res = request('get', '/files/%d/' % file_id, token=token)

    if api_res.status_code != 200:
        click.echo('Invalid file', err=True)
        exit(1)

    if api_res.json().get('category') not in ('zip', 'presentation', ''):
        click.echo('Invalid category', err=True)
        exit(1)

    return file_id



import os
import tempfile
from os import walk
from os.path import join, exists, isdir, relpath
from shutil import move, rmtree, copyfileobj
from zipfile import ZipFile

import click
import requests

from kudu.api import request as api_request
from kudu.config import ConfigOption
from kudu.types import PitcherFileType


def unpack_url(url):
    tmphandle, tmppath = tempfile.mkstemp(suffix='.zip')
    tmpfile = os.fdopen(tmphandle, 'r+b')

    res = requests.get(url, stream=True)

    copyfileobj(res.raw, tmpfile)
    tmpfile.close()

    with ZipFile(tmppath, 'r') as z:
        z.extractall()

    os.remove(tmppath)


def _move(src, dst):
    for root, dirs, files in walk(src):
        for name in files:
            arcroot = join(dst, relpath(root, src))
            if not exists(arcroot):
                os.makedirs(arcroot)
            move(join(root, name), join(arcroot, name))
    rmtree(src)


def to_dir(url, root_dir, base_dir, file_category):
    save_cwd = os.getcwd()
    os.chdir(root_dir)

    unpack_url(url)

    if exists(base_dir):
        _move(base_dir, os.curdir if file_category else 'interface')

    thumb_filename = base_dir + '.png'
    if exists(thumb_filename):
        os.rename(thumb_filename, 'thumbnail.png')

    os.chdir(save_cwd)


def to_file(download_url, path):
    res = requests.get(download_url, stream=True)
    with open(path, 'w+b') as f:
        copyfileobj(res.raw, f)


@click.command()
@click.option(
    '--file',
    '-f',
    'pf',
    cls=ConfigOption,
    config_name='file_id',
    prompt='File ID',
    type=PitcherFileType()
)
@click.option('--path', '-p', type=click.Path(), default=lambda: os.getcwd())
@click.pass_context
def pull(ctx, pf, path):
    download_url = api_request(
        'get', '/files/%d/download-url/' % pf['id'], token=ctx.obj['token']
    ).json()

    if isdir(path):
        filename_root, filename_ext = os.path.splitext(pf['filename'])

        if filename_ext == '.zip':
            to_dir(download_url, path, filename_root, pf['category'])
        else:
            to_file(download_url, join(path, pf['filename']))
    else:
        to_file(download_url, path)



from setuptools import find_packages, setup

from kudu import __author__, __email__, __version__

setup(
    name='kudu',
    author=__author__,
    author_email=__email__,
    version=__version__,
    description='A deployment command line program in Python.',
    url='https://github.com/torfeld6/kudu',
    license='BSD',
    classifiers=[
        'Intended Audience :: Developers',
        'Topic :: Utilities',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
    ],
    keywords='cli',
    packages=find_packages(),
    install_requires=[
        'requests>=2.25.1',
        'PyYAML>=5.4.1',
        'watchdog>=0.10.6,<1.0.0',
        'click>=7.1.2,<8.0.0',
    ],
    entry_points={
        'console_scripts': ['kudu = kudu.__main__:cli',],
    },
)



import os
import time
from fnmatch import fnmatch
from os import getcwd
from os.path import join, split, splitext
from shutil import copyfile

import click
from watchdog.events import (
    EVENT_TYPE_CREATED, EVENT_TYPE_MODIFIED, EVENT_TYPE_MOVED,
    FileSystemEventHandler
)
from watchdog.observers import Observer

from kudu.config import ConfigOption
from kudu.defaults import default_pitcher_folders
from kudu.types import PitcherFileType


class PitcherFilePathConverter(object):

    def __init__(self, pfile):
        self.pfile = pfile

    def convert(self, relpath):
        raise NotImplementedError()


class InteractivePathConverter(PitcherFilePathConverter):
    basedir = 'zip'

    def convert(self, relpath):
        dirname = splitext(self.pfile['filename'])[0]
        if fnmatch(relpath, 'thumbnail.png'):
            relpath = dirname + '.png'
        return join(self.basedir, dirname, relpath)


class PresentationPathConverter(InteractivePathConverter):
    basedir = 'slides'

    def convert(self, relpath):
        if fnmatch(relpath, join('iPadOnly', '*')):
            relpath = os.sep.join(split(relpath)[1:])
        return super(PresentationPathConverter, self).convert(relpath)


class InterfacePathConverter(PitcherFilePathConverter):

    def convert(self, relpath):
        if fnmatch(relpath, join('interface', '*')):
            dirname = splitext(self.pfile['filename'])[0]
            relpath = join(dirname, *split(relpath)[1:])
        return relpath


class CopyFilesEventHandler(FileSystemEventHandler):

    def __init__(self, src, dst, converter):
        self.src = src
        self.dst = dst
        self.converter = converter

    def on_any_event(self, event):
        if event.event_type in [EVENT_TYPE_MODIFIED, EVENT_TYPE_MOVED,
                                EVENT_TYPE_CREATED] and not event.is_directory:
            src_path = event.src_path if event.event_type != EVENT_TYPE_MOVED else event.dest_path
            src_relpath = os.path.relpath(src_path)

            dst_relpath = self.converter.convert(src_relpath)
            dst_path = join(self.dst, dst_relpath)
            dst_dirpath = os.path.dirname(dst_path)

            if not os.path.exists(dst_dirpath):
                os.makedirs(dst_dirpath)

            click.echo("Copying file: %s" % src_relpath, nl=False)

            try:
                copyfile(src_path, dst_path)
                click.echo("\rCopying file: %s, done." % src_relpath)
            except (IOError, OSError) as e:
                # File most likely does not exist
                click.echo(e, err=True)

            click.echo("\rCopying file: %s, done." % src_relpath)


def countfiles(top):
    click.echo('Counting files', nl=False)

    count = 0
    for root, dirs, files in os.walk(top):
        count += len(files)
        click.echo('\rCounting files: %d' % count, nl=False)

    click.echo('\rCounting files: %d, done.' % count)
    return count


def copyfiles(src, dst, converter):
    total = countfiles(src)
    curr = 0

    if not os.path.exists(dst):
        os.makedirs(dst, 0o755)

    click.echo('Copying files', nl=False)

    for root, dirs, files in os.walk(src):

        for name in files:
            curr += 1
            click.echo('\rCopying files: %d/%d' % (curr, total), nl=False)

            src_path = os.path.join(root, name)
            src_relpath = os.path.relpath(src_path)

            dst_relpath = converter.convert(src_relpath)
            dst_path = join(dst, dst_relpath)
            dst_dirpath = os.path.dirname(dst_path)

            if not os.path.exists(dst_dirpath):
                os.makedirs(dst_dirpath)

            copyfile(join(root, name), dst_path)

    click.echo('\rCopying files: %d/%d, done.' % (curr, total))


def watchfiles(src, dst, converter):
    event_handler = CopyFilesEventHandler(src, dst, converter)
    observer = Observer()
    observer.schedule(event_handler, src, recursive=True)
    observer.start()
    try:
        while True:
            time.sleep(1)
    except KeyboardInterrupt:
        observer.stop()
    observer.join()


@click.command()
@click.option(
    '--file',
    '-f',
    'pfile',
    cls=ConfigOption,
    config_name='file_id',
    prompt=True,
    type=PitcherFileType(category=['zip', 'presentation', ''])
)
@click.option(
    '--pitcher-folders',
    '-pf',
    cls=ConfigOption,
    config_name='pitcher_folders',
    prompt=True,
    default=lambda: default_pitcher_folders(),
    type=click.Path(exists=True, file_okay=False, writable=True)
)
def link(pfile, pitcher_folders):
    cwd = getcwd()

    if pfile['category'] == 'zip':
        converter = InteractivePathConverter(pfile)
    elif pfile['category'] == 'presentation':
        converter = PresentationPathConverter(pfile)
    elif pfile['category'] == '':
        converter = InterfacePathConverter(pfile)
    else:
        raise NotImplementedError()

    copyfiles(cwd, pitcher_folders, converter)
    watchfiles(cwd, pitcher_folders, converter)



import os
from collections import namedtuple
from datetime import datetime

import click
import requests

from kudu.api import request as api_request
from kudu.config import ConfigOption
from kudu.mkztemp import NameRule, mkztemp
from kudu.types import PitcherFileType

CategoryRule = namedtuple('crule', ('category', 'rule'))


CATEGORY_RULES = (
    CategoryRule('',
                 NameRule((r'^interface', r'(.+)'), ('{base_name}', '{0}'))),
    CategoryRule(('presentation', 'zip'),
                 NameRule(r'^thumbnail.png', '{base_name}.png')),
    CategoryRule(('presentation', 'zip'),
                 NameRule(r'(.+)', ('{base_name}', '{0}'))),
) # yapf: disable


@click.command()
@click.option(
    '--file',
    '-f',
    'pf',
    cls=ConfigOption,
    config_name='file_id',
    prompt=True,
    type=PitcherFileType(category=('zip', 'presentation', 'json', ''))
)
@click.option('--path', '-p', type=click.Path(exists=True), default=None)
@click.pass_context
def push(ctx, pf, path):
    name = pf['filename']
    base_name, _ = os.path.splitext(name)

    url = '/files/%d/upload-url/' % pf['id']
    response = api_request('get', url, token=ctx.obj['token'])

    if path is None or os.path.isdir(path):
        rules = [c.rule for c in CATEGORY_RULES if pf['category'] in c.category]
        fp, _ = mkztemp(base_name, root_dir=path, name_rules=rules)
        data = os.fdopen(fp, 'r+b')
    else:
        data = open(path, 'r+b')

    # upload data
    requests.put(response.json(), data=data)

    # touch file
    url = '/files/%d/' % pf['id']
    json = {'creationTime': datetime.utcnow().isoformat()}
    api_request('patch', url, json=json, token=ctx.obj['token'])
